package jalview.hmmer;

import jalview.bin.Cache;
import jalview.datamodel.AlignmentI;
import jalview.datamodel.SequenceGroup;
import jalview.datamodel.SequenceI;
import jalview.gui.AlignFrame;
import jalview.gui.Preferences;
import jalview.io.DataSourceType;
import jalview.io.FileFormat;
import jalview.util.MessageManager;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.List;

import javax.swing.JOptionPane;

public class HMMBuildThread implements Runnable
{
  HMMERCommands cmds = new HMMERCommands();
  AlignFrame af;
  AlignmentI alignment;

  SequenceGroup group;

  boolean canRun = true;

  File hmmTemp = null;

  File stoTemp = null;

  long barID;
  
  public HMMBuildThread(AlignFrame af)
  {
    this.af = af;
    if (af.getViewport().getSelectionGroup() != null)
    {
      group = af.getViewport().getSelectionGroup();
    }
    alignment = af.getViewport().getAlignment();

  }

  @Override
  public void run()
  {
    barID = System.currentTimeMillis();
    af.setProgressBar(MessageManager.getString("status.running_hmmbuild"),
            barID);
    cmds.HMMERFOLDER = Cache.getProperty(Preferences.HMMER_PATH);
    if (alignment == null && group == null)
    {
      JOptionPane.showMessageDialog(af,
              MessageManager.getString("label.no_sequence_data"));
      return;
    }
    try
    {
      hmmTemp = File.createTempFile("hmm", ".hmm");
      hmmTemp.deleteOnExit();
      stoTemp = File.createTempFile("output", ".sto");
      stoTemp.deleteOnExit();
    } catch (IOException e1)
    {
      e1.printStackTrace();
    }

    try
    {
    try
    {
        List<SequenceI> seqs = alignment
                .getHMMConsensusSequences(true);
        cmds.setHmmSeqs(seqs);
        if (group != null)
        {
          SequenceI[] array = group.getSelectionAsNewSequences(alignment);

          cmds.exportData(array, stoTemp, null, null);
        }
        else
        {
          if (!alignment.isAligned())
          {
            alignment.padGaps();
          }
          cmds.exportData(alignment.getSequencesArray(), stoTemp, null,
                  null);
        }
    } catch (FileNotFoundException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();

    }
    try
    {
      runCommand();
    } catch (IOException | InterruptedException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    try
    {

      importData();
    } catch (IOException | InterruptedException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    } catch (Exception e)
    {
      e.printStackTrace();
    } finally
    {
      af.setProgressBar(MessageManager.getString("status.running_hmmbuild"),
              barID);
    }
  }

  

  
  private void runCommand() throws IOException, InterruptedException
  {
    final String command = cmds.HMMERFOLDER + cmds.HMMBUILD + cmds.NAME
            + af.getName() + cmds.SPACE + hmmTemp.getAbsolutePath()
            + cmds.SPACE + stoTemp.getAbsolutePath() + cmds.SPACE;
    cmds.runCommand(command);
  }
  
  private void importData() throws IOException, InterruptedException
  {
    cmds.addHMMConsensusSequences(af);
    af.loadJalviewDataFile(hmmTemp.getAbsolutePath(), DataSourceType.FILE,
            FileFormat.HMMER3, null);
    hmmTemp.delete();
    stoTemp.delete();
  }
  
  public boolean canRun()
  {
    return canRun;
  }
  
  
 
}
